﻿////////////////////////////////////////////////////////////////////////////
//
// Copyright (c) 2010-2025 60East Technologies Inc., All Rights Reserved.
//
// This computer software is owned by 60East Technologies Inc. and is
// protected by U.S. copyright laws and other laws and by international
// treaties.  This computer software is furnished by 60East Technologies 
// Inc. pursuant to a written license agreement and may be used, copied,
// transmitted, and stored only in accordance with the terms of such
// license agreement and with the inclusion of the above copyright notice.
// This computer software or any other copies thereof may not be provided
// or otherwise made available to any other person.
//
// U.S. Government Restricted Rights.  This computer software: (a) was
// developed at private expense and is in all respects the proprietary
// information of 60East Technologies Inc.; (b) was not developed with
// government funds; (c) is a trade secret of 60East Technologies Inc.
// for all purposes of the Freedom of Information Act; and (d) is a
// commercial item and thus, pursuant to Section 12.212 of the Federal
// Acquisition Regulations (FAR) and DFAR Supplement Section 227.7202,
// Government’s use, duplication or disclosure of the computer software
// is subject to the restrictions set forth by 60East Technologies Inc..
//
////////////////////////////////////////////////////////////////////////////

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Data;
using System.ComponentModel;
using System.Threading;
using System.Collections;
using System.Timers;
using CommandField = AMPS.Client.Fields.CommandField;

namespace AMPS.Client.Xaml
{
    public class Subscription : DependencyObject
    {
        CommandId _cachedCommandId = new CommandId();
        bool _active = false;
        int _last_row_count = 0;
        int _total_rec = 0;
        int _last_total_rec = 0;
        int _peak_rate = 0;
        DateTime _last_total_dt;
        System.Timers.Timer _rate_timer;

        public static readonly DependencyProperty LoadingProperty = DependencyProperty.Register("Loading", typeof(bool), typeof(Subscription));
        
        public bool Loading
        {
            get
            {
                return (bool)GetValue(LoadingProperty);
            }
            set
            {
                SetValue(LoadingProperty, value);
            }

        }

        public static readonly DependencyProperty ModeProperty = DependencyProperty.Register("Mode", typeof(string), typeof(Subscription));
        public string Mode
        {
            get
            {
                return (string)GetValue(ModeProperty);
            }
            set
            {
                SetValue(ModeProperty, value);
            }
        }

        public static readonly DependencyProperty TopicProperty = DependencyProperty.Register("Topic", typeof(string), typeof(Subscription));
        public string Topic
        {
            get
            {
                return (string)GetValue(TopicProperty);
            }
            set
            {
                SetValue(TopicProperty, value);
            }
        }

        public static readonly DependencyProperty ServerProperty = DependencyProperty.Register("Server", typeof(Server), typeof(Subscription));
        public Server Server
        {
            get
            {
                return (Server)GetValue(ServerProperty);
            }
            set
            {
                SetValue(ServerProperty, value);
                Server.ConnectionStateChanged += new Xaml.Server.ConnectionStateChangedEventHandler(Server_ConnectionStateChanged);
            }
        }

        public static readonly DependencyProperty FilterProperty = DependencyProperty.Register("Filter", typeof(string), typeof(Subscription));
        public string Filter
        {
            get
            {
                return (string)GetValue(FilterProperty);
            }
            set
            {
                SetValue(FilterProperty, value);
            }
        }

        public static readonly DependencyProperty OrderByProperty = DependencyProperty.Register("OrderBy", typeof(string), typeof(Subscription));
        public string OrderBy
        {
            get
            {
                return (string)GetValue(OrderByProperty);
            }
            set
            {
                SetValue(OrderByProperty, value);
            }
        }

        public static readonly DependencyProperty TopNProperty = DependencyProperty.Register("TopN", typeof(long), typeof(Subscription));
        public long TopN
        {
            get
            {
                return (long)GetValue(TopNProperty);
            }
            set
            {
                SetValue(TopNProperty, value);
            }
        }

        public static readonly DependencyProperty ErrorProperty = DependencyProperty.Register("Error", typeof(string), typeof(Subscription));
        public string Error
        {
            get
            {
                return (string)GetValue(ErrorProperty);
            }
            set
            {
                SetValue(ErrorProperty, value);
            }
        }

        public static readonly DependencyProperty FirstDTProperty = DependencyProperty.Register("FirstDT", typeof(string), typeof(Subscription));
        public string FirstDT
        {
            get
            {
                return (string)GetValue(FirstDTProperty);
            }
            set
            {
                SetValue(FirstDTProperty, value);
            }
        }

        public static readonly DependencyProperty LastDTProperty = DependencyProperty.Register("LastDT", typeof(string), typeof(Subscription));
        public string LastDT
        {
            get
            {
                return (string)GetValue(LastDTProperty);
            }
            set
            {
                SetValue(LastDTProperty, value);
            }
        }

        public static readonly DependencyProperty TableRowCountProperty = DependencyProperty.Register("TableRowCount", typeof(int), typeof(Subscription));

        public int TableRowCount
        {
            get
            {
                return (int)GetValue(TableRowCountProperty);
            }
            set
            {
                SetValue(TableRowCountProperty, value);
            }

        }

        public static readonly DependencyProperty RecRateProperty = DependencyProperty.Register("RecRate", typeof(string), typeof(Subscription));
        public string RecRate
        {
            get
            {
                return (string)GetValue(RecRateProperty);
            }
            set
            {
                SetValue(RecRateProperty, value);
            }
        }

        public static readonly DependencyProperty PeakRateProperty = DependencyProperty.Register("PeakRate", typeof(string), typeof(Subscription));
        public string PeakRate
        {
            get
            {
                return (string)GetValue(PeakRateProperty);
            }
            set
            {
                SetValue(PeakRateProperty, value);
            }
        }

        public static readonly DependencyProperty TopicMatchesProperty = DependencyProperty.Register("TopicMatches", typeof(string), typeof(Subscription));
        public string TopicMatches
        {
            get
            {
                return (string)GetValue(TopicMatchesProperty);
            }
            set
            {
                SetValue(TopicMatchesProperty, value);
            }
        }
        Client _client;
        CommandId _subId;
        
        public void Unsubscribe()
        {
            Error = "";
            if (_client != null)
            {
                if (_subId != null)
                {
                    try
                    {
                        _client.unsubscribe(_subId);
                    }
                    catch (Exception e)
                    {
                        Error = e.Message;
                        return;
                    }
                }
            }
        }

        public void ClearTable()
        {
            _table.Clear();
        }

        public void refreshSubscription()
        {
            if (DesignerProperties.GetIsInDesignMode(this) || _table == null)
            {
                return;
            }

            Unsubscribe();
            _table.Clear();
            _subId = CommandId.nextIdentifier();

            if (Server != null && !string.IsNullOrEmpty(Topic) && _active)
            {
                PeakRate = "";
                try
                {
                    _client = Server.GetClient();

                    if (Mode.Equals("Query and Subscribe"))
                    {
                        Message m = _client.allocateMessage();
                        m.Command = Message.Commands.SOWAndDeltaSubscribe;
                        m.Topic = Topic;
                        m.Filter = Filter;
                        m.BatchSize = 100;
                        m.setSubId(_subId);
                        m.setCommandId(_subId);
                        m.setQueryId(_subId);
                        m.setAckType(Message.AckTypes.Processed | Message.AckTypes.Stats);
                        m.setOptions(Message.Options.OOF);

                        _sowRequestDT = DateTime.Now;
                        this.Dispatcher.BeginInvoke(new Action(() => FirstDT = ""));
                        this.Dispatcher.BeginInvoke(new Action(() => LastDT = ""));

                        _client.send(message => UpdateSelf(message), m, 10000);
                    }
                    else
                    {
                        _sowRequestDT = DateTime.Now;
                        this.Dispatcher.BeginInvoke(new Action(() => FirstDT = ""));
                        this.Dispatcher.BeginInvoke(new Action(() => LastDT = ""));

                        _client.sow(message => UpdateSelf(message), Topic, Filter, OrderBy, null, null, 100, (int)TopN, 10000);
                    }
                }
                catch (Exception e)
                {
                    Error = e.Message;
                }
            }
        }

        void Server_ConnectionStateChanged(object sender, Server.ConnectionEventArgs e)
        {
            this.Dispatcher.BeginInvoke(new Action(() =>
            {
                switch (e.State)
                {
                    case Server.ConnectionEventArgs.States.Connected:
                        this.refreshSubscription();
                        this.Error = "";
                        break;
                    case Server.ConnectionEventArgs.States.Disconnected:
                        this.Error = "Disconnected.";
                        this.Dispatcher.BeginInvoke(new Action(() => LastDT = ""));
                        break;
                    case Server.ConnectionEventArgs.States.Reconnecting:
                        this.Error = "Disconnected, attempting to reconnect.";
                        this.Dispatcher.BeginInvoke(new Action(() => LastDT = ""));
                        break;
                }
            }));

        }

        public static readonly DependencyProperty DataProperty = DependencyProperty.Register("Data", typeof(ISOWModel), typeof(Subscription));
        ISOWModel _table = null;

        public ISOWModel Data
        {
            get
            {
                return (ISOWModel)GetValue(DataProperty);
            }
            set
            {
                _table = value;
                _table.Dispatcher = Dispatcher;
                SetValue(DataProperty, value);
                _active = true;
                refreshSubscription();
            }
        }

        protected override void OnPropertyChanged(DependencyPropertyChangedEventArgs e)
        {
            base.OnPropertyChanged(e);

            if (e.Property.Name == "Server")
            {
                Server.ConnectionStateChanged += new Xaml.Server.ConnectionStateChangedEventHandler(Server_ConnectionStateChanged);
            }
        }

        private void UpdateRowCount()
        {
            if (_table == null) return;
            if (_last_row_count != _table.Count)
            {
                this.Dispatcher.BeginInvoke(new Action(() => TableRowCount = _table.Count));
            }
            _last_row_count = _table.Count;
        }

        private void UpdateRate(object o, ElapsedEventArgs args)
        {
            DateTime current = DateTime.Now;
            int total_rec = _total_rec;
            if (_last_total_rec != 0)
            {
                TimeSpan time_diff = current.Subtract(_last_total_dt);
                int rate = (int)((double)(total_rec - _last_total_rec) / time_diff.TotalSeconds);
                this.Dispatcher.BeginInvoke(new Action(() => RecRate = rate.ToString() + "/s"));
                if (rate > _peak_rate)
                {
                    _peak_rate = rate;
                    this.Dispatcher.BeginInvoke(new Action(() => PeakRate = rate.ToString() + "/s"));
                }
            }
            else
            {
                this.Dispatcher.BeginInvoke(new Action(() => RecRate = "0/s"));
                this.Dispatcher.BeginInvoke(new Action(() => PeakRate = "0/s"));
            }
            _last_total_dt = current;
            _last_total_rec = total_rec;
        }

        delegate void UpdateRowCountDelegate();

        UpdateRowCountDelegate _updateRowCountFunc;
        DateTime _sowRequestDT;
        DateTime _sowBeginDT;
        DateTime _sowEndDT;
        
        private void UpdateSelf(Message message)
        {
            switch (message.Command)
            {
                case Message.Commands.Ack:
                    string topicMatches = Convert.ToString(message.TopicMatches);

                    Dispatcher.BeginInvoke(new Action(() => TopicMatches = topicMatches));
                    break;

                case Message.Commands.GroupBegin:
                    _sowBeginDT = DateTime.Now;
                    TimeSpan beginDiff = _sowBeginDT.Subtract(_sowRequestDT);
                    this.Dispatcher.BeginInvoke(new Action(()=>FirstDT = beginDiff.ToString("s\\.ff") + "s"));
                    this.Dispatcher.BeginInvoke(_updateRowCountFunc);

                    this.Dispatcher.BeginInvoke(new Action(()=>Loading = true));
                    
                    break;
                case Message.Commands.GroupEnd:
                    _sowEndDT = DateTime.Now;
                    UpdateRowCount();
                    TimeSpan endDiff = _sowEndDT.Subtract(_sowRequestDT);
                    this.Dispatcher.BeginInvoke(new Action(()=>LastDT = "Initial load time: " + endDiff.ToString("s\\.ff") + "s"));

                    this.Dispatcher.BeginInvoke(new Action(()=>Loading = false));
                    _table.Reset();
                    break;
                case Message.Commands.SOW:
                    ++_total_rec;
                    var field = message.getDataRaw();
                    _table.SOW(message.SowKey, field.buffer, field.position, field.length);

                    if (_total_rec % 100000 == 0)
                    {
                        _table.Reset();
                    }
                    break;
                case Message.Commands.Publish:
                    ++_total_rec;
                    UpdateRowCount();
                    var pubField = message.getDataRaw();
                    _table.Publish(message.SowKey, pubField.buffer, pubField.position, pubField.length);
                    break;
                case Message.Commands.OOF:
                    ++_total_rec;
                    UpdateRowCount();
                    _table.OOF(message.SowKey);
                    break;
            }
        }

        public Subscription()
        {
            this.Data = new ExpandoSOWModel();
            TopicMatches = "0";
            _updateRowCountFunc = delegate() { UpdateRowCount(); };

            _rate_timer = new System.Timers.Timer(1000);
            _rate_timer.Elapsed += new ElapsedEventHandler(UpdateRate);
            _rate_timer.AutoReset = true;
            _rate_timer.Interval = 1000;
            _rate_timer.Enabled = true;
        }

        public override string ToString()
        {
            return Topic;
        }

        public string GetXaml()
        {
            StringBuilder sb = new StringBuilder();
            sb.Append("<amps:Subscription x:Key=\"MySubscription\" Server=\"{StaticResource MyServer}\" ");
            if (!string.IsNullOrEmpty(this.Topic))
            {
                sb.AppendFormat("Topic=\"{0}\"", this.Topic);
            }

            if (!string.IsNullOrEmpty(this.Filter))
            {
                sb.AppendFormat(" Filter=\"{0}\"", this.Filter);
            }
            sb.Append("/>");

            return sb.ToString();
        }

    }
}
