////////////////////////////////////////////////////////////////////////////
//
// Copyright (c) 2012-2025 60East Technologies Inc., All Rights Reserved.
//
// This computer software is owned by 60East Technologies Inc. and is
// protected by U.S. copyright laws and other laws and by international
// treaties. This computer software is furnished by 60East Technologies
// Inc. pursuant to a written license agreement and may be used, copied,
// transmitted, and stored only in accordance with the terms of such
// license agreement and with the inclusion of the above copyright notice.
// This computer software or any other copies thereof may not be provided
// or otherwise made available to any other person.
//
// U.S. Government Restricted Rights.  This computer software: (a) was
// developed at private expense and is in all respects the proprietary
// information of 60East Technologies Inc.; (b) was not developed with
// government funds; (c) is a trade secret of 60East Technologies Inc.
// for all purposes of the Freedom of Information Act; and (d) is a
// commercial item and thus, pursuant to Section 12.212 of the Federal
// Acquisition Regulations (FAR) and DFAR Supplement Section 227.7202,
// Government's use, duplication or disclosure of the computer software
// is subject to the restrictions set forth by 60East Technologies Inc..
//
////////////////////////////////////////////////////////////////////////////
using System;

namespace AMPS.Client
{

    /// <summary>
    /// Called in {@link Client} when the Client unintentionally disconnects from AMPS.
    /// This interface can be used to implement your own disconnect handler. It will
    /// notify you that a Client disconnected so that an action can be taken, such as doing
    /// a reconnect.
    /// </summary>
    public interface ClientDisconnectHandler
    {
        void invoke(Client client);
    }

    /// <summary>
    /// Allows the use of an Action (i.e. lambda function with no return value) in
    /// any context where a ClientDisconnectHandler may be used.
    /// </summary>
    public class ActionDisconnectHandler : ClientDisconnectHandler
    {
        Action<Client> _action;

        /// <summary>
        /// Initializes a new instance of the ActionDisconnectHandler class.
        /// </summary>
        /// <param name="action">The Action<Client> to be executed when disconnecting a client.</param>
        public ActionDisconnectHandler(Action<Client> action)
        {
            _action = action;
        }

        /// <summary>
        /// Invokes the stored action with the provided client parameter.
        /// </summary>
        /// <param name="client">The client parameter to be passed to the stored action.</param>
        public void invoke(Client client)
        {
            _action(client);
        }
    }
}