////////////////////////////////////////////////////////////////////////////
//
// Copyright (c) 2010-2025 60East Technologies Inc., All Rights Reserved.
//
// This computer software is owned by 60East Technologies Inc. and is
// protected by U.S. copyright laws and other laws and by international
// treaties.  This computer software is furnished by 60East Technologies
// Inc. pursuant to a written license agreement and may be used, copied,
// transmitted, and stored only in accordance with the terms of such
// license agreement and with the inclusion of the above copyright notice.
// This computer software or any other copies thereof may not be provided
// or otherwise made available to any other person.
//
// U.S. Government Restricted Rights.  This computer software: (a) was
// developed at private expense and is in all respects the proprietary
// information of 60East Technologies Inc.; (b) was not developed with
// government funds; (c) is a trade secret of 60East Technologies Inc.
// for all purposes of the Freedom of Information Act; and (d) is a
// commercial item and thus, pursuant to Section 12.212 of the Federal
// Acquisition Regulations (FAR) and DFAR Supplement Section 227.7202,
// Government’s use, duplication or disclosure of the computer software
// is subject to the restrictions set forth by 60East Technologies Inc..
//
////////////////////////////////////////////////////////////////////////////
using System;
using AMPS.Client.Exceptions;

namespace AMPS.Client
{
    /// <summary>
    /// A default implementation of the Authenticator interface that is
    /// used for basic username/password authentication.
    /// </summary>
    public class DefaultAuthenticator : Authenticator
    {
        /// <summary>
        /// Called by AMPS.Client, just before the logon command is sent.
        /// </summary>
        /// <param name="username_">The current value of the username as specified in the URI.</param>
        /// <param name="currentPassword_">The current value of the password, as specified in the URI.</param>
        /// <returns>The value that should be placed into the Password header field of the logon command</returns>
        public virtual String authenticate(String username_, String currentPassword_)
        {
            return currentPassword_;
        }

        /// <summary>
        /// Called when a logon "ack" is received with a status of "retry".
        /// </summary>
        /// <remarks>
        /// AMPS will continue trying to logon as long as the server returns "retry",
        /// and this method continues to succeed.
        /// </remarks>
        /// <param name="username_">The username returned by the server's ACK message.</param>
        /// <param name="password_">The password or token returned in the server's ACK message.</param>
        /// <returns>The value that should be placed into the Password header for the next logon attempt.</returns>
        public virtual String retry(String username_, String password_)
        {
            throw new AuthenticationException("retry not implemented by the DefaultAuthenticator.");
        }

        /// <summary>
        /// Called when a logon completes successfully.
        /// </summary>
        /// <remarks>
        /// Once a logon has completed, this method is called with the username and password
        /// that caused a successful logon.
        /// </remarks>
        /// <param name="username_">The username that successfully logged on to the server.</param>
        /// <param name="password_">The password that successfully logged on to the server.</param>
        /// <param name="reason_">The reason reported for success by the server.</param>
        public virtual void completed(String username_, String password_, Message.Reasons reason_)
        {
        }
    }

}
