////////////////////////////////////////////////////////////////////////////
//
// Copyright (c) 2012-2025 60East Technologies Inc., All Rights Reserved.
//
// This computer software is owned by 60East Technologies Inc. and is
// protected by U.S. copyright laws and other laws and by international
// treaties.  This computer software is furnished by 60East Technologies
// Inc. pursuant to a written license agreement and may be used, copied,
// transmitted, and stored only in accordance with the terms of such
// license agreement and with the inclusion of the above copyright notice.
// This computer software or any other copies thereof may not be provided
// or otherwise made available to any other person.
//
// U.S. Government Restricted Rights.  This computer software: (a) was
// developed at private expense and is in all respects the proprietary
// information of 60East Technologies Inc.; (b) was not developed with
// government funds; (c) is a trade secret of 60East Technologies Inc.
// for all purposes of the Freedom of Information Act; and (d) is a
// commercial item and thus, pursuant to Section 12.212 of the Federal
// Acquisition Regulations (FAR) and DFAR Supplement Section 227.7202,
// Government's use, duplication or disclosure of the computer software
// is subject to the restrictions set forth by 60East Technologies Inc..
//
////////////////////////////////////////////////////////////////////////////

namespace AMPS.Client
{

    /// <summary>
    /// A default no-op implementation of the ClientDisconnectHandler and
    /// TransportDisconnectHandler interfaces that does nothing.
    /// </summary>
    public class DefaultDisconnectHandler : ClientDisconnectHandler,
          TransportDisconnectHandler
    {
        /// <summary>
        /// Singleton instance of the DefaultDisconnectHandler.
        /// </summary>
        public static DefaultDisconnectHandler instance = new DefaultDisconnectHandler();

        /// <summary>
        /// Constructs a new instance of DefaultDisconnectHandler.
        /// </summary>
        internal DefaultDisconnectHandler()
        { ;}

        /// <summary>
        /// No-op implementation of the invoke method for ClientDisconnectHandler.
        /// </summary>
        /// <param name="client">The client being disconnected.</param>
        public void invoke(Client client)
        { ;}

        /// <summary>
        /// No-op implementation of the invoke method for TransportDisconnectHandler.
        /// </summary>
        /// <param name="transport">The transport instance.</param>
        /// <param name="e">The exception that occurred during disconnection.</param>
        public void invoke(Transport transport, System.Exception e)
        { ;}

        /// <summary>
        /// No-op implementation of the preInvoke method.
        /// </summary>
        /// <param name="failedVersion">The version that failed.</param>
        public void preInvoke(int failedVersion)
        { ; }
    }
}
