
using System;

namespace AMPS.Client.Exceptions
{
    /// <summary>
    /// Base class for all exceptions in the AMPS client.
    /// </summary>
    public class AMPSException : Exception
    {
        private const string GeneralMessage = "A general AMPS exception has occurred.";

        /// <summary>
        /// Constructs a new AMPSException with a general error message.
        /// </summary>
        public AMPSException()
            : base(GeneralMessage)
        {
        }

        /// <summary>
        /// Constructs a new AMPSException with a supplied error message.
        /// </summary>
        /// <param name="message">The error message</param>
        public AMPSException(string message)
            : base(message)
        {
        }

        /// <summary>
        /// Constructs a new AMPSException with a supplied inner exception.
        /// </summary>
        /// <param name="innerException">The exception to encapsulate inside this AMPSException</param>
        public AMPSException(Exception innerException)
            : base(GeneralMessage, innerException)
        {
        }

        /// <summary>
        /// Constructs a new AMPSException with a supplied message and inner exception.
        /// </summary>
        /// <param name="message">The error message</param>
        /// <param name="innerException">The exception to encapsulate inside this AMPSException</param>
        public AMPSException(string message, Exception innerException)
            : base(message, innerException)
        {
        }
    }
}
