////////////////////////////////////////////////////////////////////////////
//
// Copyright (c) 2010-2025 60East Technologies Inc., All Rights Reserved.
//
// This computer software is owned by 60East Technologies Inc. and is
// protected by U.S. copyright laws and other laws and by international
// treaties.  This computer software is furnished by 60East Technologies
// Inc. pursuant to a written license agreement and may be used, copied,
// transmitted, and stored only in accordance with the terms of such
// license agreement and with the inclusion of the above copyright notice.
// This computer software or any other copies thereof may not be provided
// or otherwise made available to any other person.
//
// U.S. Government Restricted Rights.  This computer software: (a) was
// developed at private expense and is in all respects the proprietary
// information of 60East Technologies Inc.; (b) was not developed with
// government funds; (c) is a trade secret of 60East Technologies Inc.
// for all purposes of the Freedom of Information Act; and (d) is a
// commercial item and thus, pursuant to Section 12.212 of the Federal
// Acquisition Regulations (FAR) and DFAR Supplement Section 227.7202,
// Government’s use, duplication or disclosure of the computer software
// is subject to the restrictions set forth by 60East Technologies Inc..
//
////////////////////////////////////////////////////////////////////////////

namespace AMPS.Client
{
    /// <summary>
    /// An interface for receiving information on messages that you've written, but
    /// for which the server indicates are not persisted because of a failure.
    /// </summary>
    public interface FailedWriteHandlerV4
    {
        /// <summary>
        /// The AMPS instance reports that a write has failed.
        /// </summary>
        /// <param name="sequenceNumber">The sequence number of the message that could not be written.</param>
        /// <param name="operation">A value from Message.Commands (Publish, DeltaPublish, or SOWDelete).</param>
        /// <param name="topic">The topic of the message.</param>
        /// <param name="topicOffset">The index in topic where the topic bytes begin.</param>
        /// <param name="topicLength">The length of the topic, in bytes.</param>
        /// <param name="data">The data of the message.</param>
        /// <param name="dataOffset">The index in data where the data bytes begin.</param>
        /// <param name="dataLength">The length of the data, in bytes.</param>
        /// <param name="correlationId">The correlation ID for this message.</param>
        /// <param name="correlationIdOffset">The offset into the correlation ID array where the correlation ID begins.</param>
        /// <param name="correlationIdLength">The length of the correlation ID.</param>
        /// <param name="reason">A value from Message.Reasons indicating the reason for this failed write.</param>
        void failedWrite(ulong sequenceNumber, Message.Commands operation,
                byte[] topic, long topicOffset, long topicLength,
                byte[] data, long dataOffset, long dataLength,
                byte[] correlationId, long correlationIdOffset, long correlationIdLength,
                Message.Reasons reason);
    }
}
