////////////////////////////////////////////////////////////////////////////
//
// Copyright (c) 2010-2025 60East Technologies Inc., All Rights Reserved.
//
// This computer software is owned by 60East Technologies Inc. and is
// protected by U.S. copyright laws and other laws and by international
// treaties.  This computer software is furnished by 60East Technologies
// Inc. pursuant to a written license agreement and may be used, copied,
// transmitted, and stored only in accordance with the terms of such
// license agreement and with the inclusion of the above copyright notice.
// This computer software or any other copies thereof may not be provided
// or otherwise made available to any other person.
//
// U.S. Government Restricted Rights.  This computer software: (a) was
// developed at private expense and is in all respects the proprietary
// information of 60East Technologies Inc.; (b) was not developed with
// government funds; (c) is a trade secret of 60East Technologies Inc.
// for all purposes of the Freedom of Information Act; and (d) is a
// commercial item and thus, pursuant to Section 12.212 of the Federal
// Acquisition Regulations (FAR) and DFAR Supplement Section 227.7202,
// Government’s use, duplication or disclosure of the computer software
// is subject to the restrictions set forth by 60East Technologies Inc..
//
////////////////////////////////////////////////////////////////////////////
using System;

namespace AMPS.Client.Fields
{
    /// <summary>
    /// Field data for a Message which consists of an unsigned long
    /// integer value
    /// </summary>
    public class UlongField : Field
    {
        protected static byte LATIN1_ZERO = 48;

        private byte[] bytes = new byte[21]; // LOG10(2**64)<20 (+1 for sign)

        /// <summary>
        /// Gets the value of this unsigned long field as a 64-bit
        /// unsigned integer.
        /// </summary>
        /// <returns>The value as an unsigned long integer.</returns>
        /// <exception cref="NullReferenceException">Thrown when the buffer is null.</exception>
        public ulong getValue()
        {
            if (this.buffer != null)
            {
                ulong result = 0;
                for (int i = position; i < position + length; ++i)
                {
                    result = result * 10 + (ulong)(buffer[i] - LATIN1_ZERO);
                }
                return result;
            }
            else
            {
                throw new NullReferenceException("value is null");
            }
        }

        /// <summary>
        /// Sets the value of this unsigned long field using the
        /// specified unsigned long integer.
        /// </summary>
        /// <param name="v">The value to set.</param>
        public void setValue(ulong v)
        {
            int index = 0;
            if(v == 0)
            {
              bytes[0] = (byte)'0';
              set(bytes,0,1);
              return;
            }
            ulong cv = v;
            while (cv > 0) { cv /= 10; ++index; }  // TODO: Faster than Log10?
            int length = index;
            while (v > 0)
            {
                bytes[--index] = (byte)(LATIN1_ZERO + (v % 10));
                v /= 10;
            }
            set(bytes, 0, length);
        }
    }

}
