////////////////////////////////////////////////////////////////////////////
//
// Copyright (c) 2010-2025 60East Technologies Inc., All Rights Reserved.
//
// This computer software is owned by 60East Technologies Inc. and is
// protected by U.S. copyright laws and other laws and by international
// treaties.  This computer software is furnished by 60East Technologies
// Inc. pursuant to a written license agreement and may be used, copied,
// transmitted, and stored only in accordance with the terms of such
// license agreement and with the inclusion of the above copyright notice.
// This computer software or any other copies thereof may not be provided
// or otherwise made available to any other person.
//
// U.S. Government Restricted Rights.  This computer software: (a) was
// developed at private expense and is in all respects the proprietary
// information of 60East Technologies Inc.; (b) was not developed with
// government funds; (c) is a trade secret of 60East Technologies Inc.
// for all purposes of the Freedom of Information Act; and (d) is a
// commercial item and thus, pursuant to Section 12.212 of the Federal
// Acquisition Regulations (FAR) and DFAR Supplement Section 227.7202,
// Government’s use, duplication or disclosure of the computer software
// is subject to the restrictions set forth by 60East Technologies Inc..
//
////////////////////////////////////////////////////////////////////////////

using System;
using System.Collections.Generic;

namespace AMPS.Client
{

    /// <summary>
    /// A parser class which converts a FIX message into a dictionary.
    /// </summary>
    public class FIXShredder
    {
        private byte fieldSeparator;

        /// <summary>
        /// Initializes a new instance of the FIXShredder class with
        /// the specified field separator.
        /// </summary>
        /// <param name="fieldSeparator">The byte value representing
        /// the field separator.</param>
        public FIXShredder(byte fieldSeparator)
        {
            this.fieldSeparator = fieldSeparator;
        }

        /// <summary>
        /// Method used to map the FIX message into the dictionary.
        /// </summary>
        /// <param name = "fix">The FIX message to be mapped</param>
        /// <returns>The completed dictionary</returns>
        public Dictionary<int, string> toMap(string fix)
        {
            int index = 0;
            int length = fix.Length;
            var map = new Dictionary<int, string>(128);
            while (index < length - 1)
            {
                // First, read the tag
                int endTag = fix.IndexOf('=', index);
                if (endTag == -1) break;
                int endValue = fix.IndexOf((char)this.fieldSeparator, endTag + 1);
                if (endValue == -1) break;
                map[Convert.ToInt32(fix.Substring(index, endTag - index))] = fix.Substring(endTag + 1, endValue - (endTag + 1));
                index = endValue + 1;
            }
            return map;
        }
    }

}
