////////////////////////////////////////////////////////////////////////////
//
// Copyright (c) 2010-2025 60East Technologies Inc., All Rights Reserved.
//
// This computer software is owned by 60East Technologies Inc. and is
// protected by U.S. copyright laws and other laws and by international
// treaties.  This computer software is furnished by 60East Technologies
// Inc. pursuant to a written license agreement and may be used, copied,
// transmitted, and stored only in accordance with the terms of such
// license agreement and with the inclusion of the above copyright notice.
// This computer software or any other copies thereof may not be provided
// or otherwise made available to any other person.
//
// U.S. Government Restricted Rights.  This computer software: (a) was
// developed at private expense and is in all respects the proprietary
// information of 60East Technologies Inc.; (b) was not developed with
// government funds; (c) is a trade secret of 60East Technologies Inc.
// for all purposes of the Freedom of Information Act; and (d) is a
// commercial item and thus, pursuant to Section 12.212 of the Federal
// Acquisition Regulations (FAR) and DFAR Supplement Section 227.7202,
// Government’s use, duplication or disclosure of the computer software
// is subject to the restrictions set forth by 60East Technologies Inc..
//
////////////////////////////////////////////////////////////////////////////
using System;

using AMPS.Client.Exceptions;

namespace AMPS.Client
{
    /// <summary>
    /// Implements the Portocol interface. Used to parse a byte stream as
    /// a series of messages. The byte stream must use the AMPS NVFIX
    /// protocol.
    /// </summary>
    public class NVFIXProtocol : FIXProtocol
    {
        /// <summary>
        /// Initializes a new NVFIXProtocol instance.
        /// </summary>
        public NVFIXProtocol()
        {
        }

        /// <summary>
        /// Initializes a new NVFIXProtocol instance with properties
        /// </summary>
        /// <param name="props">The properties to initialize the NVFIXProtocol.</param>
        public NVFIXProtocol(Properties props)
            : base(props)
        {
        }

        /// <summary>
        /// Initializes a new NVFIXProtocol instance with specified separators.
        /// </summary>
        /// <param name="fieldSeparator">The byte value representing the field separator.</param>
        /// <param name="headerSeparator">The byte value representing the header separator.</param>
        /// <param name="messageSeparator">The byte value representing the message separator.</param>
        public NVFIXProtocol(byte fieldSeparator,
                              byte headerSeparator,
                              byte messageSeparator)
            : base(fieldSeparator, headerSeparator, messageSeparator)
        {

        }

        /// <summary>
        /// Allocates a new NVFIX message.
        /// </summary>
        /// <returns>The newly allocated NVFIX message.</returns>
        public override Message allocateMessage()
        {

            return new NVFIXMessage(this.fieldSeparator,
                  this.headerSeparator,
                  this.messageSeparator,
                  Encoding,Encoding);
        }

        /// <summary>
        /// Gets the message stream for NVFIX protocol.
        /// </summary>
        /// <returns>The message stream for NVFIX protocol.</returns>
        public override ProtocolParser getMessageStream()
        {
            return new NVFIXProtocolParser(this);
        }
    }
}
