////////////////////////////////////////////////////////////////////////////
//
// Copyright (c) 2010-2025 60East Technologies Inc., All Rights Reserved.
//
// This computer software is owned by 60East Technologies Inc. and is
// protected by U.S. copyright laws and other laws and by international
// treaties.  This computer software is furnished by 60East Technologies
// Inc. pursuant to a written license agreement and may be used, copied,
// transmitted, and stored only in accordance with the terms of such
// license agreement and with the inclusion of the above copyright notice.
// This computer software or any other copies thereof may not be provided
// or otherwise made available to any other person.
//
// U.S. Government Restricted Rights.  This computer software: (a) was
// developed at private expense and is in all respects the proprietary
// information of 60East Technologies Inc.; (b) was not developed with
// government funds; (c) is a trade secret of 60East Technologies Inc.
// for all purposes of the Freedom of Information Act; and (d) is a
// commercial item and thus, pursuant to Section 12.212 of the Federal
// Acquisition Regulations (FAR) and DFAR Supplement Section 227.7202,
// Government's use, duplication or disclosure of the computer software
// is subject to the restrictions set forth by 60East Technologies Inc..
//
////////////////////////////////////////////////////////////////////////////

namespace AMPS.Client
{
    /// <summary>
    /// Interface used to indicate whether a Store implementation is allowed to
    /// grow (or shrink) to the requested size. If the request to resize is denied,
    /// a Store implementation may fail the operation that resulted in the request
    /// to grow by throwing an exception.
    /// </summary>
    public interface PublishStoreResizeHandler
    {

        /// <summary>
        /// Called by the Store when it wants to attempt a resize. Rather
        /// than allow a resize request to grow the store, and implementation could
        /// choose to flush the store (to allow space to be freed up) at the expense
        /// of publishing performance. If the store's unpersisted count doesn't
        /// decrease after a flush times out, it could indicate the client is disconnected
        /// or a server-side issue such as a synchronous replication destination is down.
        /// </summary>
        /// <param name="store">The store sending the resize request.</param>
        /// <param name="size">The new size the store would like to resize to.</param>
        /// <returns>A boolean indicating whether the Store is allowed to resize or not.</returns>
        bool invoke(Store store, long size);
    }
}