///////////////////////////////////////////////////////////////////////////
//
// Copyright (c) 2010-2025 60East Technologies Inc., All Rights Reserved.
//
// This computer software is owned by 60East Technologies Inc. and is
// protected by U.S. copyright laws and other laws and by international
// treaties.  This computer software is furnished by 60East Technologies
// Inc. pursuant to a written license agreement and may be used, copied,
// transmitted, and stored only in accordance with the terms of such
// license agreement and with the inclusion of the above copyright notice.
// This computer software or any other copies thereof may not be provided
// or otherwise made available to any other person.
//
// U.S. Government Restricted Rights.  This computer software: (a) was
// developed at private expense and is in all respects the proprietary
// information of 60East Technologies Inc.; (b) was not developed with
// government funds; (c) is a trade secret of 60East Technologies Inc.
// for all purposes of the Freedom of Information Act; and (d) is a
// commercial item and thus, pursuant to Section 12.212 of the Federal
// Acquisition Regulations (FAR) and DFAR Supplement Section 227.7202,
// Government’s use, duplication or disclosure of the computer software
// is subject to the restrictions set forth by 60East Technologies Inc..
//
////////////////////////////////////////////////////////////////////////////

using System;
using System.IO;
using System.Runtime.CompilerServices;
using System.Text;
using System.Threading;
using System.Collections.Generic;
using AMPS.Client.Fields;
using AMPS.Client.Exceptions;

/** Interface to provide a recovery point for a bookmark subscription. Used in
 * the RecoveryPointAdapter interface for updates and recovery.
 */
namespace AMPS.Client
{

    /// <summary>
    /// Represents a recovery point with a sequence number.
    /// </summary>
    public abstract class SequencedRecoveryPoint : RecoveryPoint
    {
        static ulong nextSequence = 0;
        static SequencedRecoveryPoint()
        {
            nextSequence = 0;
        }

        private ulong _sequence = 0;

        //Return the generation count for this recovery point
        internal ulong getSequence()
        {
            return _sequence;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="SequencedRecoveryPoint"/> class.
        /// </summary>
        public SequencedRecoveryPoint()
        {
            _sequence = nextSequence++;
        }

        internal SequencedRecoveryPoint(ulong sequence_)
        {
            _sequence = sequence_;
        }

        /// <summary>
        /// Gets the subscription ID associated with this recovery point.
        /// </summary>
        /// <returns>The subscription ID.</returns>
        public abstract Field getSubId();

        /// <summary>
        /// Gets the bookmark associated with this recovery point.
        /// </summary>
        /// <returns>The bookmark.</returns>
        public abstract BookmarkField getBookmark();

        /// <summary>
        /// Creates a copy of this recovery point.
        /// </summary>
        /// <returns>A copy of the recovery point.</returns>
        public abstract RecoveryPoint copy();

    }
}