////////////////////////////////////////////////////////////////////////////
//
// Copyright (c) 2012-2025 60East Technologies Inc., All Rights Reserved.
//
// This computer software is owned by 60East Technologies Inc. and is
// protected by U.S. copyright laws and other laws and by international
// treaties.  This computer software is furnished by 60East Technologies
// Inc. pursuant to a written license agreement and may be used, copied,
// transmitted, and stored only in accordance with the terms of such
// license agreement and with the inclusion of the above copyright notice.
// This computer software or any other copies thereof may not be provided
// or otherwise made available to any other person.
//
// U.S. Government Restricted Rights.  This computer software: (a) was
// developed at private expense and is in all respects the proprietary
// information of 60East Technologies Inc.; (b) was not developed with
// government funds; (c) is a trade secret of 60East Technologies Inc.
// for all purposes of the Freedom of Information Act; and (d) is a
// commercial item and thus, pursuant to Section 12.212 of the Federal
// Acquisition Regulations (FAR) and DFAR Supplement Section 227.7202,
// Government’s use, duplication or disclosure of the computer software
// is subject to the restrictions set forth by 60East Technologies Inc..
//
////////////////////////////////////////////////////////////////////////////
using System;

namespace AMPS.Client
{

    /// <summary>
    /// Represents a simple read buffer used for reading byte data.
    /// </summary>
    public class SimpleReadBuffer
    {
        /// <summary>
        /// Initializes a new instance of the SimpleReadBuffer class.
        /// </summary>
        public SimpleReadBuffer()
        {
            Position = 0;
            Length = 0;
            Buffer = null;
        }

        /// <summary>
        /// Initializes a new instance of the SimpleReadBuffer class with the specified
        /// buffer and length.
        /// </summary>
        /// <param name="buf">The buffer.</param>
        /// <param name="length">The length of the buffer.</param>
        public SimpleReadBuffer(byte[] buf, int length)
        {
            Reset(buf, 0, length);
        }

        /// <summary>
        /// Resets the SimpleReadBuffer with the specified buffer, position, and length.
        /// </summary>
        /// <param name="buf">The buffer.</param>
        /// <param name="pos">The position within the buffer.</param>
        /// <param name="length">The length of the buffer.</param>
        public void Reset(byte[] buf, int pos, int length)
        {
            Buffer = buf;
            Length = length;
            Position = pos;
        }

        /// <summary>
        /// Gets or sets the position within the buffer.
        /// </summary>
        public int Position;

        /// <summary>
        /// Gets or sets the length of the buffer.
        /// </summary>
        public int Length;

        /// <summary>
        /// Gets or sets the buffer.
        /// </summary>
        public byte[] Buffer;

        /// <summary>
        /// Reads a byte from the buffer and advances the position.
        /// </summary>
        /// <returns>The byte read from the buffer.</returns>
        public byte ReadByte()
        {
            return Buffer[Position++];
        }

        /// <summary>
        /// Peeks at the next byte in the buffer without advancing the position.
        /// </summary>
        /// <returns>The next byte in the buffer.</returns>
        public byte PeekByte()
        {
            return Buffer[Position];
        }

        /// <summary>
        /// Peeks at the byte at the specified position in the buffer.
        /// </summary>
        /// <param name="position">The position to peek at.</param>
        /// <returns>The byte at the specified position.</returns>
        public byte PeekByte(int position)
        {
            return Buffer[position];
        }

        /// <summary>
        /// Puts a byte at the specified index in the buffer.
        /// </summary>
        /// <param name="index">The index at which to put the byte.</param>
        /// <param name="b">The byte to put.</param>
        public void put(int index, byte b)
        {
            Buffer[index] = b;
        }

    }
}
