////////////////////////////////////////////////////////////////////////////
//
// Copyright (c) 2010-2025 60East Technologies Inc., All Rights Reserved.
//
// This computer software is owned by 60East Technologies Inc. and is
// protected by U.S. copyright laws and other laws and by international
// treaties.  This computer software is furnished by 60East Technologies
// Inc. pursuant to a written license agreement and may be used, copied,
// transmitted, and stored only in accordance with the terms of such
// license agreement and with the inclusion of the above copyright notice.
// This computer software or any other copies thereof may not be provided
// or otherwise made available to any other person.
//
// U.S. Government Restricted Rights.  This computer software: (a) was
// developed at private expense and is in all respects the proprietary
// information of 60East Technologies Inc.; (b) was not developed with
// government funds; (c) is a trade secret of 60East Technologies Inc.
// for all purposes of the Freedom of Information Act; and (d) is a
// commercial item and thus, pursuant to Section 12.212 of the Federal
// Acquisition Regulations (FAR) and DFAR Supplement Section 227.7202,
// Government’s use, duplication or disclosure of the computer software
// is subject to the restrictions set forth by 60East Technologies Inc..
//
////////////////////////////////////////////////////////////////////////////


using AMPS.Client.Exceptions;

namespace AMPS.Client
{
    /// <summary>
    /// Interface used to manage all subscriptions in AMPS. This includes:
    /// subscribing, unsubscribing and resubscribing.
    /// </summary>
    public interface SubscriptionManager
    {
        /// <summary>
        /// Subscribe to a message using the provided message handler and message.
        /// </summary>
        /// <param name="messageHandler">The message handler to handle incoming messages.</param>
        /// <param name="message">The message to subscribe to.</param>
        void subscribe(MessageHandler messageHandler, Message message);

        /// <summary>
        /// Unsubscribe from a subscription using the provided subscription ID.
        /// </summary>
        /// <param name="subId"></param>
        void unsubscribe(CommandId subId);

        /// <summary>
        /// Clear all subscriptions.
        /// </summary>
        void clear();

        /// <summary>
        /// Resubscribe to subscriptions using the provided client.
        /// </summary>
        /// <param name="client">The client to use for resubscribing.</param>
        void resubscribe(Client client);

        /// <summary>
        /// Set the handler for failed resubscribe attempts.
        /// </summary>
        /// <param name="failureHandler">The handler for failed resubscribe attempts.</param>
        void setFailedResubscribeHandler(FailedResubscribeHandler failureHandler);

        /// <summary>
        /// Get the handler for failed resubscribe attempts.
        /// </summary>
        /// <returns>The handler for failed resubscribe attempts.</returns>
        FailedResubscribeHandler getFailedResubscribeHandler();

    }
}
