﻿////////////////////////////////////////////////////////////////////////////
//
// Copyright (c) 2012-2025 60East Technologies Inc., All Rights Reserved.
//
// This computer software is owned by 60East Technologies Inc. and is
// protected by U.S. copyright laws and other laws and by international
// treaties.  This computer software is furnished by 60East Technologies
// Inc. pursuant to a written license agreement and may be used, copied,
// transmitted, and stored only in accordance with the terms of such
// license agreement and with the inclusion of the above copyright notice.
// This computer software or any other copies thereof may not be provided
// or otherwise made available to any other person.
//
// U.S. Government Restricted Rights.  This computer software: (a) was
// developed at private expense and is in all respects the proprietary
// information of 60East Technologies Inc.; (b) was not developed with
// government funds; (c) is a trade secret of 60East Technologies Inc.
// for all purposes of the Freedom of Information Act; and (d) is a
// commercial item and thus, pursuant to Section 12.212 of the Federal
// Acquisition Regulations (FAR) and DFAR Supplement Section 227.7202,
// Government's use, duplication or disclosure of the computer software
// is subject to the restrictions set forth by 60East Technologies Inc..
//
////////////////////////////////////////////////////////////////////////////using System;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net;
using System.Net.Security;
using System.Net.Sockets;
using System.Security.Authentication;
using System.Security.Cryptography.X509Certificates;
using System.Text;

namespace AMPS.Client
{
    /// <summary>
    /// The SSL Socket implementation of {@link TCPSTransport}. TCPS Transport
    /// delegates to an instance of this class. This is used internally by the
    /// client's transport. There is usually no reason to make direct use of it.
    /// Its functionality is best accessed via the client instance.
    /// </summary>
    public class TCPSTransportImpl : TCPTransportImpl
    {
        /// <summary>
        /// Constructs a TCPSTransportImpl object for the given Protocol and Properties.
        /// </summary>
        /// <param name="protocol">The protocol for communication with AMPS.</param>
        /// <param name="properties">The properties to configure the transport.</param>
        public TCPSTransportImpl(Protocol protocol, Properties properties)
            : base(protocol, properties)
        {
        }
        /// <summary>
        /// Creates an SSL stream for the specified URI.
        /// </summary>
        /// <param name="uri">The URI to create the stream for.</param>
        protected override void createStream(Uri uri)
        {
            SslStream sslStream = new SslStream(
                new HandleTimeoutStream(new NetworkStream(_socket),this),
                false,
                TCPSTransport.RemoteCertificateValidationCallback
                );

            String host = uri.Host;
            URIProperties properties = new URIProperties(uri);
            if (properties != null && properties.ContainsKey("sni"))
            {
                host = properties["sni"];
            }

            while (true) {
                try {
                    sslStream.AuthenticateAsClient(host,
                        new X509CertificateCollection(),
                        TCPSTransport.EnabledSslProtocols,
                        TCPSTransport.CheckCertificateRevocation);
                    break;
                }
                // SslProtocols.None requires registry key settings in .NET Framework < 4.7
                // so an exception will be thrown if that isn't set up properly.
                catch(System.ArgumentException) {
                    // Start with explicit Tls12, highest available in framework 4.62
                    if (TCPSTransport.EnabledSslProtocols == 0) {
                        TCPSTransport.EnabledSslProtocols = SslProtocols.Tls12;
                    }
                    else {
                        TCPSTransport.EnabledSslProtocols = (SslProtocols)(((int)TCPSTransport.EnabledSslProtocols)/4);
                    }
                    if ((int)TCPSTransport.EnabledSslProtocols <= 12 /* Ssl2 */) {
                        throw;
                    }
                }
            }
            _socketStream = sslStream;
        }
    }
}
