////////////////////////////////////////////////////////////////////////////
//
// Copyright (c) 2012-2025 60East Technologies Inc., All Rights Reserved.
//
// This computer software is owned by 60East Technologies Inc. and is
// protected by U.S. copyright laws and other laws and by international
// treaties.  This computer software is furnished by 60East Technologies
// Inc. pursuant to a written license agreement and may be used, copied,
// transmitted, and stored only in accordance with the terms of such
// license agreement and with the inclusion of the above copyright notice.
// This computer software or any other copies thereof may not be provided
// or otherwise made available to any other person.
//
// U.S. Government Restricted Rights.  This computer software: (a) was
// developed at private expense and is in all respects the proprietary
// information of 60East Technologies Inc.; (b) was not developed with
// government funds; (c) is a trade secret of 60East Technologies Inc.
// for all purposes of the Freedom of Information Act; and (d) is a
// commercial item and thus, pursuant to Section 12.212 of the Federal
// Acquisition Regulations (FAR) and DFAR Supplement Section 227.7202,
// Government's use, duplication or disclosure of the computer software
// is subject to the restrictions set forth by 60East Technologies Inc..
//
////////////////////////////////////////////////////////////////////////////
using System;
using System.Threading;

namespace AMPS.Client
{
    /// <summary>
    /// Interface for classes that handle AMPS messages.
    /// </summary>
    public interface ThreadCreatedHandler
    {
        /// <summary>
        /// Method for the AMPS transport to call when a thread is created.
        /// <p>
        /// The thread calling invoke is the newly created thread.
        /// </summary>
        void invoke();
    }

    /// <summary>
    /// Allows usage of an Action (i.e. a lambda function with no return value)
    /// in the place of a MessageHandler throughout the AMPS api.
    /// </summary>
    public class ActionThreadCreatedHandler : ThreadCreatedHandler
    {
        Action _action;

        /// <summary>
        /// Initializes a new instance of the <see cref="ActionThreadCreatedHandler"/> class.
        /// </summary>
        /// <param name="action">The action to be invoked.</param>
        public ActionThreadCreatedHandler(Action action)
        {
            _action = action;
        }

        /// <summary>
        /// Method for the AMPS transport to call when a thread is created.
        /// <p>
        /// The thread calling invoke is the newly created thread.
        /// </summary>
        public void invoke()
        {
            _action();
        }
    }
}