////////////////////////////////////////////////////////////////////////////
//
// Copyright (c) 2010-2025 60East Technologies Inc., All Rights Reserved.
//
// This computer software is owned by 60East Technologies Inc. and is
// protected by U.S. copyright laws and other laws and by international
// treaties.  This computer software is furnished by 60East Technologies
// Inc. pursuant to a written license agreement and may be used, copied,
// transmitted, and stored only in accordance with the terms of such
// license agreement and with the inclusion of the above copyright notice.
// This computer software or any other copies thereof may not be provided
// or otherwise made available to any other person.
//
// U.S. Government Restricted Rights.  This computer software: (a) was
// developed at private expense and is in all respects the proprietary
// information of 60East Technologies Inc.; (b) was not developed with
// government funds; (c) is a trade secret of 60East Technologies Inc.
// for all purposes of the Freedom of Information Act; and (d) is a
// commercial item and thus, pursuant to Section 12.212 of the Federal
// Acquisition Regulations (FAR) and DFAR Supplement Section 227.7202,
// Government’s use, duplication or disclosure of the computer software
// is subject to the restrictions set forth by 60East Technologies Inc..
//
////////////////////////////////////////////////////////////////////////////
using System;
/// <summary>
/// Filter raw incoming and outgoing data on a Transport.
/// </summary>
public class TransportFilterWrapper : TransportFilterModifiable
{
    private TransportFilter _filter = null;

    /// <summary>
    /// Initializes a new instance of the TransportFilterWrapper class with
    /// the specified filter.
    /// </summary>
    /// <param name="filter_"></param>
    public TransportFilterWrapper(TransportFilter filter_)
    {
        _filter = filter_;
    }

    /// <summary>
    /// Filters outgoing data.
    /// </summary>
    /// <param name="data">The data to filter.</param>
    /// <param name="position">The starting position in the data.</param>
    /// <param name="length">The length of the data to filter.</param>
    public void outgoing(ref byte[] data, int position, int length)
    {
        _filter.outgoing((byte[])data, position, length);
    }

    /// <summary>
    /// Filters incoming data.
    /// </summary>
    /// <param name="data">The data to filter.</param>
    /// <param name="position">The starting position in the data.</param>
    /// <param name="length">The length of the data to filter.</param>
    public void incoming(ref byte[] data, int position, int length)
    {
        _filter.incoming((byte[])data, position, length);
    }
}
