////////////////////////////////////////////////////////////////////////////
//
// Copyright (c) 2010-2025 60East Technologies Inc., All Rights Reserved.
//
// This computer software is owned by 60East Technologies Inc. and is
// protected by U.S. copyright laws and other laws and by international
// treaties.  This computer software is furnished by 60East Technologies
// Inc. pursuant to a written license agreement and may be used, copied,
// transmitted, and stored only in accordance with the terms of such
// license agreement and with the inclusion of the above copyright notice.
// This computer software or any other copies thereof may not be provided
// or otherwise made available to any other person.
//
// U.S. Government Restricted Rights.  This computer software: (a) was
// developed at private expense and is in all respects the proprietary
// information of 60East Technologies Inc.; (b) was not developed with
// government funds; (c) is a trade secret of 60East Technologies Inc.
// for all purposes of the Freedom of Information Act; and (d) is a
// commercial item and thus, pursuant to Section 12.212 of the Federal
// Acquisition Regulations (FAR) and DFAR Supplement Section 227.7202,
// Government’s use, duplication or disclosure of the computer software
// is subject to the restrictions set forth by 60East Technologies Inc..
//
////////////////////////////////////////////////////////////////////////////
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

namespace AMPS.Client
{
    /// <summary>
    /// An implementation of TransportFilter that traces to a System.IO.TextWriter.
    /// To use, instantiate and hen pass to Transport's setTransportFilter, for example:
    /// <code>
    ///   Client client = new Client(...);
    ///   client.connect(...);
    ///   client.getTransport.setTransportFilter(
    ///     new TransportTraceFilter(Console.out)
    ///   );
    ///   </code>
    /// </summary>
    public class TransportTraceFilter : TransportFilter
    {
        TextWriter _writer;
        /// <summary>
        /// Construct a TransportTraceFilter to trace messages on writer_.
        /// </summary>
        /// <param name="writer">A System.IO.TextWriter to trace on.</param>
        public TransportTraceFilter(TextWriter writer_)
        {
            _writer = writer_;
        }

        /// <summary>
        /// Function to print the raw data.
        /// </summary>
        /// <param name="prefix">The prefix label passed to the function. This will be
        /// either incoming or outgoing.</param>
        /// <param name="data">The bytebuffer of raw data to be printed.</param>
        /// <param name="position">The starting position of the data.</param>
        /// <param name="length">The length of the data.</param>
        protected void print(String prefix, byte[] data, int position, int length)
        {
            _writer.Write(prefix);
            _writer.WriteLine(Encoding.UTF8.GetString(data, position, length));
        }

        /// <summary>
        /// Method to call the prrint() method with the outgoing prefix for printing
        /// raw data.
        /// </summary>
        /// <param name="data">The outgoing data.</param>
        /// <param name="position">The starting position of the data.</param>
        /// <param name="length">The length of the data.</param>
        public void outgoing(byte[] data, int position, int length)
        {
            print("OUTGOING ===> ", data, position, length);
        }

        /// <summary>
        /// Method to call the print() method with the incoming prefix for printing raw
        /// data.
        /// </summary>
        /// <param name="data">The incoming data.</param>
        /// <param name="position">The starting position of the data.</param>
        /// <param name="length">The length of the data.</param>
        public void incoming(byte[] data, int position, int length)
        {
            print("INCOMING ===> ", data, position, length);
        }
    }
}
