////////////////////////////////////////////////////////////////////////////
//
// Copyright (c) 2010-2025 60East Technologies Inc., All Rights Reserved.
//
// This computer software is owned by 60East Technologies Inc. and is
// protected by U.S. copyright laws and other laws and by international
// treaties.  This computer software is furnished by 60East Technologies
// Inc. pursuant to a written license agreement and may be used, copied,
// transmitted, and stored only in accordance with the terms of such
// license agreement and with the inclusion of the above copyright notice.
// This computer software or any other copies thereof may not be provided
// or otherwise made available to any other person.
//
// U.S. Government Restricted Rights.  This computer software: (a) was
// developed at private expense and is in all respects the proprietary
// information of 60East Technologies Inc.; (b) was not developed with
// government funds; (c) is a trade secret of 60East Technologies Inc.
// for all purposes of the Freedom of Information Act; and (d) is a
// commercial item and thus, pursuant to Section 12.212 of the Federal
// Acquisition Regulations (FAR) and DFAR Supplement Section 227.7202,
// Government’s use, duplication or disclosure of the computer software
// is subject to the restrictions set forth by 60East Technologies Inc..
//
////////////////////////////////////////////////////////////////////////////
using System;

namespace AMPS.Client
{
    /// <summary>
    /// Provides a Properties interface to the parameters passed in a URI without depending on
    /// System.Web.
    /// </summary>
    public class URIProperties : Properties
    {
        /// <summary>
        /// Initializes a new instance of the URIProperties class with the specified URI.
        /// </summary>
        /// <param name="uri_"></param>
        public URIProperties(Uri uri_)
        {
            parse(uri_);
        }

        private void parse(Uri uri_)
        {
            this.Clear();
            string query = uri_.Query.TrimStart('?');
            int currentKey = 0, currentKeyLen = 0, currentValue = 0;
            for(int i = 0; i < query.Length; i++)
            {
                if(currentValue == 0) { // parsing the key
                    if(query[i] == '=') {
                        currentValue = i+1;
                        currentKeyLen = i-currentKey;
                    }
                } else {
                    if(query[i] == '&') {
                        this.Add(Uri.UnescapeDataString(query.Substring(currentKey, currentKeyLen)),
                                  Uri.UnescapeDataString(query.Substring(currentValue, i - currentValue)));
                        currentKey = i+1;
                        currentValue = 0;
                    }
                }
            }
            if(currentValue > currentKey) { // last one
                this.Add(Uri.UnescapeDataString(query.Substring(currentKey, currentKeyLen)),
                        Uri.UnescapeDataString(query.Substring(currentValue)));
            }
        }
    }
}

