﻿using System;
using System.Threading;

using AMPS.Client;
using AMPS.Client.Fields;
using AMPS.Client.Exceptions;
// using AMPS.Client.ServerChooser;

// AMPSConsolePublisher
//
// This is a minimalist way of publishing messages to a topic in
// AMPS using HAClient.
// The program flow is simple:
//
// * Connect to AMPS
// * Logon
// * Publish a message the "messages" topic
//
// This sample doesn't include robust error handling.
//
// This file is a part of the AMPS Evaluation Kit.
////////////////////////////////////////////////////////////////////////////
//
// Copyright (c) 2013-2025 60East Technologies Inc., All Rights Reserved.
//
// This computer software is owned by 60East Technologies Inc. and is
// protected by U.S. copyright laws and other laws and by international
// treaties.  This computer software is furnished by 60East Technologies
// Inc. pursuant to a written license agreement and may be used, copied,
// transmitted, and stored only in accordance with the terms of such
// license agreement and with the inclusion of the above copyright notice.
// This computer software or any other copies thereof may not be provided
// or otherwise made available to any other person.
//
// U.S. Government Restricted Rights.  This computer software: (a) was
// developed at private expense and is in all respects the proprietary
// information of 60East Technologies Inc.; (b) was not developed with
// government funds; (c) is a trade secret of 60East Technologies Inc.
// for all purposes of the Freedom of Information Act; and (d) is a
// commercial item and thus, pursuant to Section 12.212 of the Federal
// Acquisition Regulations (FAR) and DFAR Supplement Section 227.7202,
// Government’s use, duplication or disclosure of the computer software
// is subject to the restrictions set forth by 60East Technologies Inc..
//
////////////////////////////////////////////////////////////////////////////

namespace AMPSConsolePublisher
{
    class AMPSConsolePublisher
    {

        private static string uri_ = "tcp://127.0.0.1:9027/amps/json";

        static void Main(string[] args)
        {
            Console.WriteLine("Starting the console publisher.");

            using(HAClient client = new HAClient("examplePublisher"))
            {
                try
                {
                    // Wait a bit to give a subscriber time to listen
                    Thread.Sleep(2000);

                    // Create a custom server chooser
                    DefaultServerChooser sc = new DefaultServerChooser();
                    sc.add(uri_);

                    // Set the server chooser for the HAClient
                    client.setServerChooser(sc);

                    // Connect to the AMPS server and logon
                    client.connectAndLogon();
                    Console.WriteLine("Publisher Connected");

                    // publish a simple JSON message to the "messages" topic
                    client.publish("messages", "{ \"hi\" : \"Hello, World!\" }");
                    Console.WriteLine("Published a message.");

                    // Wait up to 4 seconds for all messages to be acknowledged by AMPS
                    client.publishFlush(4000);
                }
                catch (AMPSException e)
                {
                    Console.Error.WriteLine(e.Message);
                }
            }

        }
    }
}
