﻿using System;

using AMPS.Client;
using AMPS.Client.Exceptions;

// AMPSSOWConsolePublisher
//
// This sample publishes messages to a topic in AMPS that
// maintains a state-of-the-world (SOW) database.
// The program flow is simple:
//
// * Connect to AMPS
// * Logon
// * Publish 100 messages to the "messages-sow" topic
// * Publish another message with a duplicate messageNumber to the topic,
//   effectively updating that message.
//
// The "messages-sow" topic is configured in config/sample.xml to
// maintain a SOW database, where each messageNumber is a unique
// message.
//
// This sample doesn't include robust error handling.
//
// This file is a part of the AMPS Evaluation Kit.
////////////////////////////////////////////////////////////////////////////
//
// Copyright (c) 2013-2025 60East Technologies Inc., All Rights Reserved.
//
// This computer software is owned by 60East Technologies Inc. and is
// protected by U.S. copyright laws and other laws and by international
// treaties.  This computer software is furnished by 60East Technologies
// Inc. pursuant to a written license agreement and may be used, copied,
// transmitted, and stored only in accordance with the terms of such
// license agreement and with the inclusion of the above copyright notice.
// This computer software or any other copies thereof may not be provided
// or otherwise made available to any other person.
//
// U.S. Government Restricted Rights.  This computer software: (a) was
// developed at private expense and is in all respects the proprietary
// information of 60East Technologies Inc.; (b) was not developed with
// government funds; (c) is a trade secret of 60East Technologies Inc.
// for all purposes of the Freedom of Information Act; and (d) is a
// commercial item and thus, pursuant to Section 12.212 of the Federal
// Acquisition Regulations (FAR) and DFAR Supplement Section 227.7202,
// Government’s use, duplication or disclosure of the computer software
// is subject to the restrictions set forth by 60East Technologies Inc..
//
////////////////////////////////////////////////////////////////////////////

namespace AMPSConsoleSOWPublisher
{
    class AMPSSOWConsolePublisher
    {
        private static string uri_ = "tcp://127.0.0.1:9027/amps/json";

        static void Main(string[] args)
        {
            using(HAClient client = new HAClient("exampleSOWPublisher"))
            {
                try
                {
                    DefaultServerChooser sc = new DefaultServerChooser();
                    sc.add(uri_);

                    client.setServerChooser(sc);

                    client.connectAndLogon();
                    Console.WriteLine("Publisher Connected");
                    // send 100 messages with unique message numbers
                    // to fill the SOW database
                    for (int number = 0; number < 100; ++number)
                    {
                        client.publish("messages-sow",
                                       "{ \"text\" : \"Hello, world!\", " +
                                       " \"messageNumber\" : " + number + "}");
                    }

                    // Now make a change to message 5.

                    client.publish("messages-sow", "{ \"text\":\"This is new information\", " +
                                                      "\"messageNumber\" : 5 }" );

                    // Wait up to 4 seconds for all messages to be acknowledged by AMPS
                    client.publishFlush(4000);
                }
                catch (AMPSException e)
                {
                    Console.Error.WriteLine(e.Message);
                }
            }
        }
    }

}
