﻿using System;
using System.Threading;

using AMPS.Client;
using AMPS.Client.Exceptions;

// AMPSConsoleSubscriber
//
// This is a minimalist way of subscribing to a topic in AMPS. The
// program flow is simple:
//
// * Connect to AMPS
// * Logon
// * Subscribe to all messages published on the "messages" topic
// * Output the messages to the console
//
// This sample doesn't include robust error handling.
//
// This file is a part of the AMPS Evaluation Kit.
////////////////////////////////////////////////////////////////////////////
//
// Copyright (c) 2013-2025 60East Technologies Inc., All Rights Reserved.
//
// This computer software is owned by 60East Technologies Inc. and is
// protected by U.S. copyright laws and other laws and by international
// treaties.  This computer software is furnished by 60East Technologies
// Inc. pursuant to a written license agreement and may be used, copied,
// transmitted, and stored only in accordance with the terms of such
// license agreement and with the inclusion of the above copyright notice.
// This computer software or any other copies thereof may not be provided
// or otherwise made available to any other person.
//
// U.S. Government Restricted Rights.  This computer software: (a) was
// developed at private expense and is in all respects the proprietary
// information of 60East Technologies Inc.; (b) was not developed with
// government funds; (c) is a trade secret of 60East Technologies Inc.
// for all purposes of the Freedom of Information Act; and (d) is a
// commercial item and thus, pursuant to Section 12.212 of the Federal
// Acquisition Regulations (FAR) and DFAR Supplement Section 227.7202,
// Government’s use, duplication or disclosure of the computer software
// is subject to the restrictions set forth by 60East Technologies Inc..
//
////////////////////////////////////////////////////////////////////////////

namespace AMPSConsoleSubscriber
{
    class AMPSConsoleSubscriber
    {

        private static string uri_ = "tcp://127.0.0.1:9027/amps/json";

        static void Main(string[] args)
        {

            using (HAClient client = new HAClient("exampleSubscriber"))
            {
                CommandId subscriptionId = new CommandId();

                try
                {

                    DefaultServerChooser sc = new DefaultServerChooser();
                    sc.add(uri_);
                    client.setServerChooser(sc);

                    client.connectAndLogon();

                    // Subscribe to the messages topic. When a message arrives,
                    // write the message data to the console.

                    foreach (var message in client.execute(
                                     new Command(Message.Commands.Subscribe)
                                           .setTopic("messages")))
                    {
                        Console.WriteLine(message.Data);
                    }


                }
                catch (AMPSException e)
                {
                    Console.Error.WriteLine(e.Message);
                }
            }
        }
    }
}
