﻿using System;
using System.Text;

using AMPS.Client;
using AMPS.Client.Exceptions;

// AMPSFIXBuilderPublisher
//
// This is a minimalist way of publishing FIX messages to a topic in AMPS.
// The program flow is simple:
//
// * Connect to AMPS
// * Logon
// * Create a FIX builder and append data
// * Publish a message the "messages" topic
//
// This sample doesn't include robust error handling.
//
// This file is a part of the AMPS Evaluation Kit.
////////////////////////////////////////////////////////////////////////////
//
// Copyright (c) 2013-2025 60East Technologies Inc., All Rights Reserved.
//
// This computer software is owned by 60East Technologies Inc. and is
// protected by U.S. copyright laws and other laws and by international
// treaties.  This computer software is furnished by 60East Technologies
// Inc. pursuant to a written license agreement and may be used, copied,
// transmitted, and stored only in accordance with the terms of such
// license agreement and with the inclusion of the above copyright notice.
// This computer software or any other copies thereof may not be provided
// or otherwise made available to any other person.
//
// U.S. Government Restricted Rights.  This computer software: (a) was
// developed at private expense and is in all respects the proprietary
// information of 60East Technologies Inc.; (b) was not developed with
// government funds; (c) is a trade secret of 60East Technologies Inc.
// for all purposes of the Freedom of Information Act; and (d) is a
// commercial item and thus, pursuant to Section 12.212 of the Federal
// Acquisition Regulations (FAR) and DFAR Supplement Section 227.7202,
// Government’s use, duplication or disclosure of the computer software
// is subject to the restrictions set forth by 60East Technologies Inc..
//
////////////////////////////////////////////////////////////////////////////

namespace AMPSFIXBuilderPublisher
{
    class AMPSFIXBuilderPublisher
    {

        private static string uri_ = "tcp://127.0.0.1:9027/amps/fix";

        static void Main(string[] args)
        {
            using (HAClient client = new HAClient("exampleFIXPublisher"))
            {
                try
                {
                    DefaultServerChooser sc = new DefaultServerChooser();
                    sc.add(uri_);
                    client.setServerChooser(sc);
                    client.connectAndLogon();

                    // Create a FIXBuilder with 1024 bytes of initial capacity
                    FIXBuilder builder = new FIXBuilder(1024, (byte)1);

                    // Add fields to the builder
                    builder.append(0, "data");
                    builder.append(1, "more data");

                    byte[] topicBytes = Encoding.ASCII.GetBytes("messages");
                    byte[] messageBytes = builder.getBytes();

                    // Print the message to the console
                    Console.WriteLine("Message to be published:");

                    // Publish the message to the "messages" topic
                    client.publish(topicBytes, 0, topicBytes.Length, messageBytes, 0, builder.getSize());

                    Console.WriteLine(BitConverter.ToString(messageBytes, 0, builder.getSize()));
                    Console.WriteLine("Published a FIX message as a byte array.");

                    // Wait up to 4 seconds for all messages to be acknowledged by AMPS
                    client.publishFlush(4000);

                }
                catch (AMPSException e)
                {
                    Console.Error.WriteLine(e.Message);
                }
                catch (Exception ex)
                {
                    Console.Error.WriteLine("General Exception: " + ex.Message);
                }
            }

        }
    }
}

