﻿using System;
using System.Text;
using System.Threading;
using System.Collections.Generic;
using AMPS.Client;
using AMPS.Client.Exceptions;

// AMPSFIXShredderSubscriber
//
// This sample retrieves fix messages from a state-of-the-world database.
// The program flow is simple:
//
// * Connect to AMPS
// * Logon
// * Get the state-of-the-world for the "messages" topic
// * Output all fields of the messages received to the console
//
// This sample doesn't include robust error handling.
//
// This file is a part of the AMPS Evaluation Kit.
////////////////////////////////////////////////////////////////////////////
//
// Copyright (c) 2013-2025 60East Technologies Inc., All Rights Reserved.
//
// This computer software is owned by 60East Technologies Inc. and is
// protected by U.S. copyright laws and other laws and by international
// treaties.  This computer software is furnished by 60East Technologies
// Inc. pursuant to a written license agreement and may be used, copied,
// transmitted, and stored only in accordance with the terms of such
// license agreement and with the inclusion of the above copyright notice.
// This computer software or any other copies thereof may not be provided
// or otherwise made available to any other person.
//
// U.S. Government Restricted Rights.  This computer software: (a) was
// developed at private expense and is in all respects the proprietary
// information of 60East Technologies Inc.; (b) was not developed with
// government funds; (c) is a trade secret of 60East Technologies Inc.
// for all purposes of the Freedom of Information Act; and (d) is a
// commercial item and thus, pursuant to Section 12.212 of the Federal
// Acquisition Regulations (FAR) and DFAR Supplement Section 227.7202,
// Government’s use, duplication or disclosure of the computer software
// is subject to the restrictions set forth by 60East Technologies Inc..
//
////////////////////////////////////////////////////////////////////////////

namespace AMPSFIXShredderSubscriber
{
    class AMPSFIXShredderSubscriber
    {

        private static string uri_ = "tcp://127.0.0.1:9027/amps/fix";

        static void Main(string[] args)
        {
            using (HAClient client = new HAClient("exampleFIXClient"))
            {

                try
                {
                    DefaultServerChooser sc = new DefaultServerChooser();
                    sc.add(uri_);
                    client.setServerChooser(sc);
                    client.connectAndLogon();

                    string topic = "messages";

                    // Subscribe to the "messages" topic using a string
                    MessageStream ms = client.subscribe(topic);

                    try
                    {
                        // Create a shredder to parse FIX messages
                        FIXShredder shredder = new FIXShredder((byte)1);

                        // Iterate through each message and write data to console
                        foreach (Message msg in ms)
                        {
                            Console.WriteLine("Got a message");

                            // Shred the message to a map
                            Dictionary<int, string> fields = shredder.toMap(msg.getData());

                            // Iterate over the keys in the map and display the key and data
                            foreach (KeyValuePair<int, string> key in fields)
                            {
                                Console.WriteLine($"Field {key.Key}: {key.Value}");
                            }
                        }
                    }
                    finally // Close the message stream to release the subscription
                    {
                        ms.close();
                    }
                }
                catch (AMPSException e)
                {
                    Console.Error.WriteLine(e);
                }
            }
        }
    }
}
