﻿using System;
using System.Text;

using AMPS.Client;
using AMPS.Client.Exceptions;

// AMPSNVFIXBuilderPublisher
//
// This is a minimalist way of publishing NVFIX messages to a topic in AMPS.
// The program flow is simple:
//
// * Connect to AMPS
// * Logon
// * Create a NVFIX builder and append data
// * Publish a message the "messages-sow" topic
//
// This sample doesn't include error handling.
//
// This file is a part of the AMPS Evaluation Kit.
////////////////////////////////////////////////////////////////////////////
//
// Copyright (c) 2013-2025 60East Technologies Inc., All Rights Reserved.
//
// This computer software is owned by 60East Technologies Inc. and is
// protected by U.S. copyright laws and other laws and by international
// treaties.  This computer software is furnished by 60East Technologies
// Inc. pursuant to a written license agreement and may be used, copied,
// transmitted, and stored only in accordance with the terms of such
// license agreement and with the inclusion of the above copyright notice.
// This computer software or any other copies thereof may not be provided
// or otherwise made available to any other person.
//
// U.S. Government Restricted Rights.  This computer software: (a) was
// developed at private expense and is in all respects the proprietary
// information of 60East Technologies Inc.; (b) was not developed with
// government funds; (c) is a trade secret of 60East Technologies Inc.
// for all purposes of the Freedom of Information Act; and (d) is a
// commercial item and thus, pursuant to Section 12.212 of the Federal
// Acquisition Regulations (FAR) and DFAR Supplement Section 227.7202,
// Government’s use, duplication or disclosure of the computer software
// is subject to the restrictions set forth by 60East Technologies Inc..
//
////////////////////////////////////////////////////////////////////////////

namespace AMPSNVFIXBuilderPublisher
{
    class AMPSNVFIXBuilderPublisher
    {

        private static string uri_ = "tcp://127.0.0.1:9027/amps/nvfix";

        static void Main(string[] args)
        {
            using (HAClient client = new HAClient("exampleNVFIXPublisher"))
            {
                try
                {

                    DefaultServerChooser sc = new DefaultServerChooser();
                    sc.add(uri_);
                    client.setServerChooser(sc);

                    client.connectAndLogon();

                    // create a builder with 1024 bytes of initial capacity
                    // using the default 0x01 delimiter
                    NVFIXBuilder builder = new NVFIXBuilder(1024, (byte)1);

                    // add fields to the builder
                    builder.append("test", "data");
                    builder.append("more", "test data");

                    // create a byte array for the topic
                    byte[] topic = Encoding.ASCII.GetBytes("messages-sow");
                    byte[] messageBytes = builder.getBytes();

                    // publish the message to the "messages-sow" topic
                    client.publish(topic, 0, topic.Length, messageBytes, 0, builder.getSize());

                    // Display the message bytes
                    Console.WriteLine(BitConverter.ToString(messageBytes, 0, builder.getSize()));

                    Console.WriteLine("Published a message.");

                    // Wait up to 4 seconds for all messages to be acknowledged by AMPS
                    client.publishFlush(4000);
                }
                catch (AMPSException e)
                {
                    Console.Error.WriteLine(e.Message);
                }
            }

        }
    }
}
